# -*- coding: utf-8 -*-

import requests
import boto3

def get_external_ip():
      SERVICE_URLS = ['http://checkip.amazonaws.com/']
      for url in SERVICE_URLS:
            response = requests.get(url)
            return response.text.strip()

def get_sg_id_from_name(sg_name):
      ec2 = boto3.client('ec2')
      groups = ec2.describe_security_groups(Filters=[{
            'Name': 'group-name',
            'Values': [sg_name]
            },
            ]).get('SecurityGroups', [])
      if len(groups) > 1:
            raise Exception('More than one SecurityGroup found with name: %s' % sg_name)
      elif len(groups) == 0:
            raise Exception('No SecurityGroups found with name: %s' % sg_name)
      return groups[0].get('GroupId')

def get_all_sgs():
      ec2 = boto3.resource('ec2')
      return ec2.security_groups.all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from awsesame.utils import get_external_ip, get_sg_id_from_name
import boto3

def __open_one_sg(sg_name):
      sg_id = get_sg_id_from_name(sg_name)
      ec2 = boto3.resource('ec2')
      current_ip = get_external_ip()
      current_cidr = current_ip + "/32"
      security_group = ec2.SecurityGroup(sg_id)
      security_group.authorize_ingress(
                  IpProtocol='tcp',
                  FromPort=22,
                  ToPort=22,
                  CidrIp=current_cidr
                  )

def __open_all_sgs():
      ec2 = boto3.resource('ec2')
      print("Opening incoming port 22 on ALL Security Groups.")
      print("I hope you know what you're doing.")
      for sg in ec2.security_groups.all():
            print("Opening port 22 on SG %s" % sg.group_name)
            # security_group.authorize_ingress(
                        # IpProtocol='tcp',
                        # FromPort=22,
                        # ToPort=22,
                        # CidrIp=current_cidr
                        # )

def sg_open(sg_name=None, all_sgs=False):
      if all_sgs:
            __open_all_sgs()
      if sg_name is not None:
            __open_one_sg(sg_name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import six
import sys

from argparse import ArgumentParser
from awsesame.sg_open import sg_open
from awsesame.sg_close import sg_close
from awsesame.sg_list import print_open_sgs, print_closed_sgs

def create_parser():
      parser = ArgumentParser()
      subparsers = parser.add_subparsers(dest='command')
      p_open = subparsers.add_parser('open', help='Add rules to open SSH to the local IP')
      open_group = p_open.add_mutually_exclusive_group(required=True)
      open_group.add_argument('--name', dest='name', help='Name of a Securiy Group to which to add the SSH rule')
      open_group.add_argument('-a', '--all', action='store_true', help='If set, will add the SSH rule to all existing Security Groups')

      p_close = subparsers.add_parser('close', help='Removes rules allowing SSH access to the local IP')
      close_group = p_close.add_mutually_exclusive_group(required=True)
      close_group.add_argument('--name', dest='name', help='Name of a Securiy Group from which to remove the SSH rules')
      close_group.add_argument('-a', '--all', action='store_true', help='If set, will try to remove all SSH rules from a given SG, or all SSH rules specific to the local IP, otherwise')

      subparsers.add_parser('list-open', help='Lists Security Groups that have SSH rules allowing access from this IP')
      subparsers.add_parser('list-closed', help='Lists Security Groups that do not have SSH rules allowing access from this IP')

      return parser


def main():
      if six.PY2:
            print("Python 2 is not supported. Please Run this script with python 3 if you would.")
            sys.exit(1)

      parser = create_parser()
      if len(sys.argv) == 1:
            parser.print_help()
            sys.exit(0)

      args = parser.parse_args()
      if args.command == 'close':
            sg_close(sg_name=args.name, all_sgs=args.all)
      elif args.command == 'open':
            sg_open(sg_name=args.name, all_sgs=args.all)
      elif args.command == 'list-open':
            print_open_sgs()
      elif args.command == 'list-closed':
            print_closed_sgs()


if __name__ == "__main__":
      main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from awsesame.utils import get_external_ip, get_sg_id_from_name, get_all_sgs
from ipaddress import IPv4Address
from awsesame.sg_list import get_open_permissions
import boto3

def sg_close(sg_name=None, all_sgs=False):
      if all_sgs:
            sgs = get_all_sgs()
      else:
            ec2 = boto3.resource('ec2')
            sg_id = get_sg_id_from_name(sg_name)
            sgs = [ec2.SecurityGroup(sg_id)]
      
      current_ip = IPv4Address(get_external_ip())
      for security_group in sgs:
            permissions = get_open_permissions(security_group, current_ip)
            if permissions:
                  print('Closing SSH to this IP on Security Group %s via permissions:' % security_group.group_name)
                  security_group.revoke_ingress(IpPermissions=permissions)
                  for permission in permissions:
                        print(permission)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='aws-open',
            version='1.0.0',
            author='Guilherme Victal',
            author_email='guilherme at victal.eti.br',
            url='https://github.com/victal/aws-open',
            description='A small script to easily open and close external SSH on AWS Security groups.',
            license='MIT',
            packages=['awsesame'],
            requires=['six', 'requests', 'boto3', 'ipaddress'],
            entry_points={
                  'console_scripts': [
                        'awsesame=awsesame.cli:main'
                        ]
                  },
            classifiers=[
                  # How mature is this project? Common values are
                  #   3 - Alpha
                  #   4 - Beta
                  #   5 - Production/Stable
                  'Development Status :: 3 - Alpha',

                  # Indicate who your project is intended for
                  'Intended Audience :: Developers',
                  'Topic :: Software Development :: Build Tools',

                  # Pick your license as you wish (should match "license" above)
                  'License :: OSI Approved :: MIT License',

                  # Specify the Python versions you support here. In particular, ensure
                  # that you indicate whether you support Python 2, Python 3 or both.
                  'Programming Language :: Python :: 3',
                  'Programming Language :: Python :: 3.2',
                  'Programming Language :: Python :: 3.3',
                  'Programming Language :: Python :: 3.4',
                  'Programming Language :: Python :: 3.5',
                  ]
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import boto3
from ipaddress import IPv4Network, IPv4Address
from awsesame.utils import get_external_ip

def list_open_sgs():
      ec2 = boto3.resource('ec2')
      current_ip = IPv4Address(get_external_ip())
      open_sgs = []
      for sg in ec2.security_groups.all():
            perms = get_open_permissions(sg, current_ip)
            if perms:
                  open_sgs.append((sg, perms))

      return open_sgs

def list_closed_sgs():
      ec2 = boto3.resource('ec2')
      current_ip = IPv4Address(get_external_ip())
      closed_sgs = []
      for sg in ec2.security_groups.all():
           if not get_open_permissions(sg, current_ip):
                  closed_sgs.append(sg)

      return closed_sgs


def print_open_sgs():
      tuples = list_open_sgs()
      if not tuples:
            print("No open Security groups found")
      for sg, permissions in tuples:
            print("%s open due to permissions:" % sg.group_name)
            for permission in permissions:
                  print(permission)

def print_closed_sgs():
      sgs = list_closed_sgs()
      if not sgs:
            print("No closed Security groups found")
      for sg in sgs:
            print(sg.group_name)

def get_open_permissions(security_group, ip_address):
      permissions = []
      for permission in security_group.ip_permissions:
            if permission.get('IpProtocol') == 'tcp' and \
               22 in range(permission.get('FromPort'), permission.get('ToPort') + 1):
                     for ip_range in permission.get('IpRanges'):
                           network = IPv4Network(ip_range.get('CidrIp'))
                           if ip_address in network:
                                 permissions.append(permission)
      return permissions
